-- vim: ts=4 sw=4 ai et enc=utf8 fenc=utf8

-- Icons used to indicate status
local NORMAL_ICON = "Interface\\Icons\\Ability_Seal"
local HEROIC_ICON = "Interface\\Icons\\Spell_Shadow_DemonicFortitude"

-- Add-on setup
local ldb = LibStub:GetLibrary ("LibDataBroker-1.1")
local DungeonDiff = CreateFrame ("Frame", "Broker_DungeonDifficulty")

-- Support function to find our display frame
local our_frame = nil
local function cache_frame (frame) our_frame = frame end

DungeonDiff.dataobj = ldb:NewDataObject ("Dungeon Difficulty", { type = 'data source', text = "", label = "Difficulty", icon = NORMAL_ICON, OnEnter = cache_frame })

-- Event driver
DungeonDiff:SetScript ("OnEvent", function(self, event, ...) 
	if self[event] then 
		return self[event](self, event, ...) 
	end 
end)

-- Status display support
local map = {
    [0] = { -- Party mappings
        [1] = { -- Normal
            icon = NORMAL_ICON,
            value = 5,
            text = DUNGEON_DIFFICULTY_5PLAYER,
        },
        [2] = { -- Heroic
            icon = HEROIC_ICON,
            value = 5,
            text = DUNGEON_DIFFICULTY_5PLAYER_HEROIC,
        }
    },
    [1] = { -- Raid mappings
        [1] = { -- 10-man Normal
            icon = NORMAL_ICON,
            value = 10,
            text = RAID_DIFFICULTY_10PLAYER,
        },
        [2] = { -- 25-man Normal
            icon = NORMAL_ICON,
            value = 25,
            text = RAID_DIFFICULTY_25PLAYER,
        },
        [3] = { -- 10-man Heroic
            icon = HEROIC_ICON,
            value = 10,
            text = RAID_DIFFICULTY_10PLAYER_HEROIC,
        },
        [4] = { -- 25-man Heroic
            icon = HEROIC_ICON,
            value = 25,
            text = RAID_DIFFICULTY_25PLAYER_HEROIC,
        }
    }
}

-- Hide Blizzard's minimap instance difficulty indicator
local HideBlizzardIcon = true --needs to be user-modifiable and saved across sessions if this should be toggleable
if HideBlizzardIcon then
   if MiniMapInstanceDifficulty then
      MiniMapInstanceDifficulty:UnregisterAllEvents()
      MiniMapInstanceDifficulty:Hide()
   end
end

function DungeonDiff:DoRefresh()
    local nr = GetNumRaidMembers ()
    local sel
    if nr > 0 then
        sel = map[1][GetRaidDifficulty ()]
    else
        sel = map[0][GetDungeonDifficulty ()]
    end
    self.dataobj.icon = sel.icon
    self.dataobj.text = sel.text
    self.dataobj.value = sel.value
end

DungeonDiff.PLAYER_ENTERING_WORLD = DungeonDiff.DoRefresh
DungeonDiff.CHAT_MSG_SYSTEM = DungeonDiff.DoRefresh
DungeonDiff.PARTY_MEMBERS_CHANGED = DungeonDiff.DoRefresh
DungeonDiff.RAID_ROSTER_UPDATE = DungeonDiff.DoRefresh
DungeonDiff.PARTY_CONVERTED_TO_RAID = DungeonDiff.DoRefresh

DungeonDiff:RegisterEvent("PLAYER_ENTERING_WORLD")
DungeonDiff:RegisterEvent("CHAT_MSG_SYSTEM")
DungeonDiff:RegisterEvent("PARTY_MEMBERS_CHANGED")
DungeonDiff:RegisterEvent("RAID_ROSTER_UPDATE")
DungeonDiff:RegisterEvent("PARTY_CONVERTED_TO_RAID")

-- Drop down menu support
DungeonDiff.data = {}
DungeonDiff.data.menu = {
    {   text = DUNGEON_DIFFICULTY, isTitle = 1, notCheckable = 1 },

    {   text = DUNGEON_DIFFICULTY_5PLAYER,
        func = function() SetDungeonDifficulty (1) end,
        checked = function() return GetDungeonDifficulty () == 1 end },

    {   text = DUNGEON_DIFFICULTY_5PLAYER_HEROIC,
        func = function() SetDungeonDifficulty (2) end,
        checked = function() return GetDungeonDifficulty () == 2 end },

    {   text = "", isTitle = 1, notCheckable = 1 },

    {   text = RAID_DIFFICULTY, isTitle = 1, notCheckable = 1 },

    {   text = RAID_DIFFICULTY_10PLAYER,
        func = function() SetRaidDifficulty (1) end,
        checked = function() return GetRaidDifficulty () == 1 end },

    {   text = RAID_DIFFICULTY_25PLAYER,
        func = function() SetRaidDifficulty (2) end,
        checked = function() return GetRaidDifficulty () == 2 end },

    {   text = RAID_DIFFICULTY_10PLAYER_HEROIC,
        func = function() SetRaidDifficulty (3) end,
        checked = function() return GetRaidDifficulty () == 3 end },

    {   text = RAID_DIFFICULTY_25PLAYER_HEROIC,
        func = function() SetRaidDifficulty (4) end,
        checked = function() return GetRaidDifficulty () == 4 end },

    {   text = "", isTitle = 1, notCheckable = 1 },

    {   text = CLOSE,
        func = function() CloseDropDownMenus() end,
        notCheckable = 1 },
}

DungeonDiff.menu = CreateFrame("Frame", "DungeonDiffMenu", UIParent, "UIDropDownMenuTemplate")

function DungeonDiff.dataobj:OnClick ()
    if not our_frame then return end -- This shouldn't happen, but better safe than cause a Lua error
    EasyMenu (DungeonDiff.data.menu, DungeonDiff.menu, our_frame:GetName (), 0, 0, "MENU")
end


